@@ -1,13 +1,4 @@
 (**************************************************************)
-(*
- *  Ensemble, 1.10
- *  Copyright 2001 Cornell University, Hebrew University
- *  All rights reserved.
- *
- *  See ensemble/doc/license.txt for further information.
- *)
-(**************************************************************)
-(**************************************************************)
 (* TIMESTAMP.ML *)
 (* Author: Mark Hayden, 4/97 *)
 (**************************************************************)
@@ -184,6 +175,4 @@ let print () =
     eprintf "TIMESTAMP:tenths=%s words\n" (string_of_int_array words) ;
   )
 
-let _ =
-  Elink.put Elink.timestamp_add linker_add ;
-  Elink.put Elink.timestamp_print print
+
