@@ -7,29 +7,52 @@
 
 #ifndef __CE_THREADS_H__
 #define __CE_THREADS_H__
+#include "ce_so.h"
 
 /* An opaque value, representing a semaphore.
  */
 typedef struct ce_sema ce_sema_t;
 
+/* An opaque value, representing a lock.
+ */
+typedef struct ce_lck ce_lck_t;
+
 /* Create a new thread.  
  */
-void ce_thread_Create(void (*routine)(void*), void *arg, int stacksize);
+LINKDLL void ce_thread_Create(void (*routine)(void*), void *arg, int stacksize);
 
 /* Create a semaphore.  
  */
-ce_sema_t *ce_sema_Create(int initial_value);
+LINKDLL ce_sema_t *ce_sema_Create(int initial_value);
 
 /* Destroy a semaphore.  
  */
-void ce_sema_Destroy(ce_sema_t *sm);
+LINKDLL void ce_sema_Destroy(ce_sema_t *sm);
 
 /* Increase the value of a semaphore.  
  */
-void hot_sema_Inc(ce_sema_t *sema);
+LINKDLL void ce_sema_Inc(ce_sema_t *sema);
 
 /* Decrease the value of a semaphore (may block).  
  */
-void ce_sema_Dec(ce_sema_t *sema);
+LINKDLL void ce_sema_Dec(ce_sema_t *sema);
+
+
+/* Create a lock
+ */
+LINKDLL ce_lck_t *ce_lck_Create(void);
+
+/* Destroy a lock
+ */
+LINKDLL void ce_lck_Destroy(ce_lck_t *lck);
+
+/* Lock a lock
+ */
+LINKDLL void ce_lck_Lock(ce_lck_t *lck);
+
+/* Unlock a lock
+ */
+LINKDLL void ce_lck_Unlock(ce_lck_t *lck);
+
 
 #endif  /* __CE_THREADS_H__*/
