@@ -1,16 +1,8 @@
 (**************************************************************)
-(*
- *  Ensemble, 1.10
- *  Copyright 2001 Cornell University, Hebrew University
- *  All rights reserved.
- *
- *  See ensemble/doc/license.txt for further information.
- *)
-(**************************************************************)
-(**************************************************************)
 (* ISAAC.ML *)
 (* Author: Ohad Rodeh, 11/98 *)
 (**************************************************************)
+open Ensemble
 open Shared
 
 external isaacml_Init : Buf.t -> Prng.context
