@@ -16,12 +16,14 @@ type dn = t
 
   (* Event types *)
 type typ =
-    (* These events should have messages associated with them. *)
-  | ECast				(* Multicast message *)
-  | ESend				(* Pt2pt message *)
-  | ESubCast				(* Multi-destination message *)
-  | ECastUnrel				(* Unreliable multicast message *)
-  | ESendUnrel				(* Unreliable pt2pt message *)
+    (* These events have messages associated with them. *)
+  | ECast of Iovecl.t			(* Multicast message *)
+  | ESend of Iovecl.t			(* Pt2pt message *)
+  | ECastUnrel of Iovecl.t   	        (* Unreliable multicast message *)
+  | ESendUnrel of Iovecl.t		(* Unreliable pt2pt message *)
+
+  (* These events have Buf.t messages, not iovec's. 
+   *)
   | EMergeRequest			(* Request a merge *)
   | EMergeGranted			(* Grant a merge request *)
   | EMergeDenied			(* Deny a merge request *)
@@ -30,7 +32,6 @@ type typ =
 
     (* These types do not have messages. *)
   | EAccount				(* Output accounting information *)
-(*| EAck			      *)(* Acknowledge message *)
   | EAsync				(* Asynchronous application event *)
   | EBlock				(* Block the group *)
   | EBlockOk				(* Acknowledge blocking of group *)
@@ -69,13 +70,22 @@ type typ =
   | ERekeyCleanup
   | ERekeyCommit 
 
+
+(**************************************************************)
+(* These are compact descriptions of types. Used for optimizing
+ * headers in event.ml.
+*)
+type compact_typ = 
+  | C_ECast
+  | C_ESend
+  | C_EOther
+
 (**************************************************************)
 
 type field =
       (* Common fields *)
   | Type	of typ			(* type of the message*)
   | Peer	of rank			(* rank of sender/destination *)
-  | Iov	        of Iovecl.t		(* payload of message *)
   | ApplMsg				(* was this message generated by an appl? *)
 
       (* Uncommon fields *)
@@ -147,6 +157,11 @@ val create	: debug -> typ -> field list -> t
   (* Modifier *)
 val set		: debug -> t -> field list -> t
 
+(**************************************************************)
+
+  (* Modifier *)
+val set_typ	: debug -> t -> typ -> t
+
 (**************************************************************)
 
   (* Copier *) 
@@ -154,13 +169,9 @@ val copy	: debug -> t -> t
 
 (**************************************************************)
 
-  (* Destructor *)
+  (* Destructor, free the iovec ref-count *)
 val free	: debug -> t -> unit
 
-(* Same as above, but don't modify iovec refcounts.
- *)
-val free_noIov	: debug -> t -> unit
-
 (**************************************************************)
 
   (* Sanity checkers *)
@@ -177,9 +188,10 @@ val string_of_type : typ -> string
 
   (* Special constructors *)
 
-val bodyCore		: debug -> typ -> origin -> Iovecl.t -> t
+val bodyCore		: debug -> typ -> origin -> t
 val castEv		: debug	-> t
-val castUnrel		: debug	-> t
+val castUnrel	        : debug	-> t
+val castUnrelIov	: debug	-> Iovecl.t -> t
 val castIov		: debug -> Iovecl.t -> t
 val castIovAppl		: debug -> Iovecl.t -> t
 val castPeerIov 	: debug -> rank -> Iovecl.t -> t
@@ -187,6 +199,7 @@ val sendPeer		: debug -> rank	-> t
 val sendPeerIov         : debug -> rank -> Iovecl.t -> t
 val sendPeerIovAppl	: debug -> rank -> Iovecl.t -> t
 val sendUnrelPeer       : debug -> rank -> t
+val sendUnrelPeerIov    : debug -> rank -> Iovecl.t -> t
 val suspectReason       : debug -> bool Arrayf.t -> debug -> t
 val timerAlarm		: debug -> Time.t -> t
 val timerTime		: debug -> Time.t -> t
@@ -209,7 +222,7 @@ val getExtender     	: (field -> 'a option) -> 'a -> t -> 'a
 val getExtendOpt	: t -> (field -> bool) -> unit
 val getFailures		: t -> bool Arrayf.t
 val getFragment		: t -> bool
-val getIov		: t -> Iovecl.t
+val getIov		: t -> Iovecl.t option
 val getNoTotal		: t -> bool
 val getNoVsync          : t -> bool
 val getForceVsync       : t -> bool 
@@ -224,6 +237,7 @@ val getSuspectReason	: t -> string
 val getSuspects		: t -> bool Arrayf.t
 val getTime		: t -> Time.t
 val getType		: t -> typ
+val getCompactType	: t -> compact_typ
 val getViewState	: t -> View.state
 val getSecureMsg        : t -> Buf.t
 val getChannelList      : t -> (Trans.rank * Security.key) list
@@ -233,7 +247,8 @@ val getSecChannelList   : t -> Trans.rank list
 val getSecStat          : t -> int 
 val getRekeyFlag        : t -> bool 
 
-val setIovFragment	: debug -> t -> Iovecl.t -> t
+val setSendIovFragment  : debug -> t -> Iovecl.t -> t
+val setCastIovFragment  : debug -> t -> Iovecl.t -> t
 val setNoTotal  	: debug -> t -> t
 val getTree             : t -> bool * Tree.z 
 val getTreeAct          : t -> Tree.sent
@@ -243,6 +258,4 @@ val setSendUnrelPeer   	: debug -> t -> rank -> t
 
 (**************************************************************)
 
-val getIovLen           : t -> Buf.len
 
-(**************************************************************)
